package com.github.jfcloud.gene.flow.controller;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.github.jfcloud.common.annotation.WebApiLog;
import com.github.jfcloud.common.core.util.R;
import com.github.jfcloud.gene.constants.GeneStatusEnum;
import com.github.jfcloud.gene.flow.dto.FlowDetailDto;
import com.github.jfcloud.gene.flow.dto.FlowPageDto;
import com.github.jfcloud.gene.flow.service.impl.FlowInfoServiceImpl;
import com.github.jfcloud.gene.flow.vo.FlowDetailVo;
import com.github.jfcloud.gene.flow.vo.FlowPageVo;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import javafx.util.Pair;
import lombok.RequiredArgsConstructor;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

@Tag(name = "基因定制流程")
@RestController
@RequestMapping("/flow")
@RequiredArgsConstructor
public class FlowController {

    private final FlowInfoServiceImpl flowInfoService;

    @Operation(summary = "列表查询")
    @GetMapping("/page")
    public R<Page<FlowPageDto>> page(FlowPageVo vo) {
        return R.ok(flowInfoService.getPage(vo));
    }

    @Operation(summary = "详情")
    @GetMapping("/{id}")
    public R<FlowDetailDto> detail(@PathVariable Long id) {
        return R.ok(flowInfoService.getDetail(id));
    }

    @Operation(summary = "状态列表")
    @GetMapping("/statusList")
    public R<List<Pair<String, String>>> statusList() {
        List<Pair<String, String>> pairs = Stream.of(GeneStatusEnum.values())
                .map(x -> new Pair<>(x.getStatus(), x.getDescription()))
                .collect(Collectors.toList());
        return R.ok(pairs);
    }

    @WebApiLog
    @Operation(summary = "新增")
    @PostMapping
    public R save(@Valid @RequestBody FlowDetailVo vo) {
        flowInfoService.saveFlow(vo);
        return R.ok();
    }

    @WebApiLog
    @Operation(summary = "编辑")
    @PutMapping("/{id}")
    public R update(@PathVariable Long id, @Valid @RequestBody FlowDetailVo vo) {
        flowInfoService.updateFlow(id, vo);
        return R.ok();
    }

    @WebApiLog
    @Operation(summary = "删除")
    @DeleteMapping("/{id}")
    public R delete(@PathVariable Long id) {
        flowInfoService.removeById(id);
        return R.ok();
    }
}
